/*
	 Copyright (C) 2003 Paul Brossier

	 This program is free software; you can redistribute it and/or modify
	 it under the terms of the GNU General Public License as published by
	 the Free Software Foundation; either version 2 of the License, or
	 (at your option) any later version.

	 This program is distributed in the hope that it will be useful,
	 but WITHOUT ANY WARRANTY; without even the implied warranty of
	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	 GNU General Public License for more details.

	 You should have received a copy of the GNU General Public License
	 along with this program; if not, write to the Free Software
	 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	 
*/

#include "aubio_priv.h"
#include "sample.h"
#include "mathutils.h"
#include "fft.h"

#include <fftw3.h>
#define FFTW_TYPE fftwf_complex


struct _aubio_fft_t {
	uint_t fft_size;
	uint_t channels;
	smpl_t 				*in, *out;
	fft_data_t 		*specdata;
	fftwf_plan 		pfw, pbw;
};

/* static functions called by creat/delete */
static aubio_fft_t * aubio_fft_alloc(uint_t size);
static void aubio_fft_free(aubio_fft_t *s);

/**
 * static objects to calculate fft. 
 * these works as scratch pads for all fft executions
 */

static aubio_fft_t * aubio_fft_alloc(uint_t size) {
	aubio_fft_t * s = AUBIO_NEW(aubio_fft_t);
	/* allocate memory */
	s->in				= AUBIO_ARRAY(smpl_t,size);
	s->specdata	= (fft_data_t*)fftwf_malloc(sizeof(fft_data_t)*size);
	s->out			= AUBIO_ARRAY(smpl_t,size);
	return s;
}

static void aubio_fft_free(aubio_fft_t * s) { 
	/* destroy data */
	fftwf_destroy_plan(s->pfw);
	fftwf_destroy_plan(s->pbw);
	if (s->specdata) 	fftwf_free(s->specdata);
	if (s->out)				AUBIO_FREE(s->out);
	if (s->in )				AUBIO_FREE(s->in );
}

aubio_fft_t * new_aubio_fft(uint_t size) {
	aubio_fft_t * s =(aubio_fft_t *)aubio_fft_alloc(size);
	/* create plans */
	s->pfw = fftwf_plan_dft_r2c_1d(size, s->in,  s->specdata, FFTW_ESTIMATE);
	s->pbw = fftwf_plan_dft_c2r_1d(size, s->specdata, s->out, FFTW_ESTIMATE);
	return s;
}

void del_aubio_fft(aubio_fft_t * s) {
	aubio_fft_free(s);
}

void aubio_fft_do(const aubio_fft_t * s, 
		const smpl_t * data, fft_data_t * spectrum, 
		const uint_t size) {
	uint_t i;
	for (i=0;i<size;i++) s->in[i] = data[i];
	fftwf_execute(s->pfw);
	for (i=0;i<size;i++) spectrum[i] = s->specdata[i];
}

void aubio_fft_rdo(const aubio_fft_t * s, 
		const fft_data_t * spectrum, 
		smpl_t * data, 
		const uint_t size) {
	uint_t i;
	const smpl_t renorm = 1./(smpl_t)size;
	for (i=0;i<size;i++) s->specdata[i] = spectrum[i];
	fftwf_execute(s->pbw);
	for (i=0;i<size;i++) data[i] = s->out[i]*renorm;
}


void aubio_fft_getnorm(smpl_t * norm, fft_data_t * spectrum, uint_t size) {
  uint_t i;
  uint_t tmpsize = (uint_t)FLOOR(size / 2.) ;
  for (i=0;i<tmpsize;i++) {
    norm[i]      = ABSC(spectrum[i]);
    //norm[size-i] = norm[i];
  }
  //norm[tmpsize]=0.;
}

void aubio_fft_getphas(smpl_t * phas, fft_data_t * spectrum, uint_t size) {
  uint_t i;
  uint_t tmpsize = (uint_t)FLOOR(size / 2.) ;
  for (i=0;i<tmpsize;i++) {
    phas[i] 			= +ARGC(spectrum[i]);
    //phas[size-i]  = -phas[i];
  }
  //phas[tmpsize]=PI;
}

